#****************************************************************************
#
# Copyright (c) 2006 - 2007 Novell, Inc. All Rights Reserved.
# THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
# NO PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED COPIED, DISTRIBUTED,
# REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED, EXPANDED, COLLECTED,
# COMPILED, LINKED, RECAST, TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN
# CONSENT OF NOVELL, INC. ANY USE OR EXPLOITATION OF THIS WORK WITHOUT
# AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.
#
#  Author:		Novell, Inc.
#  Workfile:	CIMClient.pm
#  Contents:	Novell BCC CIM Client class
#
#****************************************************************************

package Novell_BCC::CIMClient;

#**********************************************************************
#	Use Statements
#**********************************************************************
use strict;
use warnings;
use diagnostics;
use Getopt::Long;
use Novell_BCC;

require Exporter;
our @ISA = ("Exporter");
our @EXPORT_OK = ("EnumInstances",
				  "EnumInstanceNames",
				  "InvokeMethod",
				  "Associators",
				  "CreateCIMValue",
				  "AddNamedValue",
				  "CreateObjectPath",
				  "CompareObjectPaths",
                  "ERR_SUCCESS",
                  "ERR_UNKNOWN",
                  "ERR_INVALID_XML",
                  "ERR_INVALID_EDIR_RELATIONSHIP",
                  "ERR_INVALID_OBJECT",
                  "ERR_INVALID_RESOURCE_STATE",
                  "ERR_NOT_BCC_ENABLED",
				  "ERR_INVALID_PARAMETER",
                  "ERR_NO_MEMORY",
                  "ERR_VFS_FILE_IO",
                  "ERR_BUFFER_TO_SMALL",
                  "ERR_DSML_READ",
                  "ERR_DSML_MODIFY",
                  "ERR_NOT_SUPPORTED",
                  "ERR_OBTAINING_LOCK",
                  "ERR_INVALID_CREDENTIALS",
                  "ERR_NICI",
                  "ERR_NO_PEER",
                  "ERR_INVALID_API_VERSION",
                  "ERR_POOL_NOT_FOUND",
                  "ERR_NSMI",
                  "ERR_CIM_CLIENT",
                  "ERR_CREATING_SYSTEM_RESOURCE",
                  "ERR_FILE_IO",
                  "ERR_NO_DATA",
                  "ERR_NOT_A_MEMBER",
				  "ERR_INVALID_TOKEN",
				  "ERR_INVALID_CLUSTER",
				  "ERR_NSMI_SCRIPT_TOO_LONG",
				  "ERR_RESOURCE_HAS_NO_VOLUMES",
				  "CIMTYPE_NULL",
				  "CIMTYPE_SINT8",
				  "CIMTYPE_UINT16",
				  "CIMTYPE_SINT16",
				  "CIMTYPE_UINT32",
				  "CIMTYPE_SINT32",
				  "CIMTYPE_UINT64",
				  "CIMTYPE_SINT64",
				  "CIMTYPE_STRING",
				  "CIMTYPE_BOOLEAN",
				  "CIMTYPE_REAL32",
				  "CIMTYPE_REAL64",
				  "CIMTYPE_DATETIME",
				  "CIMTYPE_CHAR16",
				  "CIMTYPE_REFERENCE",
				  "CIMTYPE_EMBEDDEDCLASS",
				  "CIMTYPE_EMBEDDEDINSTANCE",
				  "CIMTYPE_MAXDATATYPE",
				  "CIMTYPE_INVALID");

#**********************************************************************
#	Exported Constants
#**********************************************************************
#**********************************************************************
#	BCC Error Codes
#**********************************************************************
use constant ERR_SUCCESS						=> scalar 0;
use constant ERR_UNKNOWN						=> scalar 1000;
use constant ERR_INVALID_XML					=> scalar 1001;
use constant ERR_INVALID_EDIR_RELATIONSHIP		=> scalar 1002;
use constant ERR_INVALID_OBJECT					=> scalar 1003;
use constant ERR_INVALID_RESOURCE_STATE			=> scalar 1004;
use constant ERR_NOT_BCC_ENABLED				=> scalar 1005;
use constant ERR_INVALID_PARAMETER				=> scalar 1006;
use constant ERR_NO_MEMORY						=> scalar 1007;
use constant ERR_VFS_FILE_IO					=> scalar 1008;
use constant ERR_BUFFER_TO_SMALL				=> scalar 1009;
use constant ERR_DSML_READ						=> scalar 1010;
use constant ERR_DSML_MODIFY					=> scalar 1011;
use constant ERR_NOT_SUPPORTED					=> scalar 1012;
use constant ERR_OBTAINING_LOCK					=> scalar 1013;
use constant ERR_INVALID_CREDENTIALS			=> scalar 1014;
use constant ERR_NICI							=> scalar 1015;
use constant ERR_NO_PEER						=> scalar 1016;
use constant ERR_INVALID_API_VERSION			=> scalar 1017;
use constant ERR_POOL_NOT_FOUND					=> scalar 1018;
use constant ERR_NSMI							=> scalar 1019;
use constant ERR_CIM_CLIENT						=> scalar 1020;
use constant ERR_CREATING_SYSTEM_RESOURCE		=> scalar 1021;
use constant ERR_FILE_IO						=> scalar 1022;
use constant ERR_NO_DATA						=> scalar 1023;
use constant ERR_NOT_A_MEMBER					=> scalar 1024;
use constant ERR_INVALID_TOKEN					=> scalar 1025;
use constant ERR_INVALID_CLUSTER				=> scalar 1026;
use constant ERR_NSMI_SCRIPT_TOO_LONG			=> scalar 1027;
use constant ERR_RESOURCE_HAS_NO_VOLUMES		=> scalar 1028;

#**********************************************************************
#	CIM Data Types
#**********************************************************************
use constant CIMTYPE_NULL						=> scalar 0;
use constant CIMTYPE_UINT8						=> scalar 1;
use constant CIMTYPE_SINT8						=> scalar 2;
use constant CIMTYPE_UINT16						=> scalar 3;
use constant CIMTYPE_SINT16						=> scalar 4;
use constant CIMTYPE_UINT32						=> scalar 5;
use constant CIMTYPE_SINT32						=> scalar 6;
use constant CIMTYPE_UINT64						=> scalar 7;
use constant CIMTYPE_SINT64						=> scalar 8;
use constant CIMTYPE_STRING						=> scalar 9;
use constant CIMTYPE_BOOLEAN					=> scalar 10;
use constant CIMTYPE_REAL32						=> scalar 11;
use constant CIMTYPE_REAL64						=> scalar 12;
use constant CIMTYPE_DATETIME					=> scalar 13;
use constant CIMTYPE_CHAR16						=> scalar 14;
use constant CIMTYPE_REFERENCE					=> scalar 15;
use constant CIMTYPE_EMBEDDEDCLASS				=> scalar 16;
use constant CIMTYPE_EMBEDDEDINSTANCE			=> scalar 17;
use constant CIMTYPE_MAXDATATYPE				=> scalar 18;
use constant CIMTYPE_INVALID					=> scalar 99;

#**********************************************************************
#	Private Constants
#**********************************************************************
use constant WBEMOP_GET_CLASS					=> scalar 1;
use constant WBEMOP_GET_CLASS_DEFINITION		=> scalar 2;
use constant WBEMOP_DELETE_CLASS				=> scalar 3;
use constant WBEMOP_ENUMERATE_CLASS				=> scalar 4;
use constant WBEMOP_ENUMERATE_CLASS_NAMES		=> scalar 5;
use constant WBEMOP_CREATE_INSTANCE				=> scalar 6;
use constant WBEMOP_MODIFY_INSTANCE				=> scalar 7;
use constant WBEMOP_ENUMERATE_INSTANCES			=> scalar 8;
use constant WBEMOP_ENUMERATE_INSTANCE_NAMES	=> scalar 9;
use constant WBEMOP_ASSOCIATE_INSTANCES			=> scalar 10;
use constant WBEMOP_ASSOCIATE_INSTANCE_NAMES	=> scalar 11;
use constant WBEMOP_REFERNECE_INSTANCES			=> scalar 12;
use constant WBEMOP_REFERNECE_INSTANCE_NAMES	=> scalar 13;
use constant WBEMOP_GET_PROPERTY				=> scalar 14;
use constant WBEMOP_SET_PROPERTY				=> scalar 15;
use constant WBEMOP_INVOKE_METHOD				=> scalar 16;
use constant WBEMOP_LOG_MESSAGE					=> scalar 17;

#**********************************************************************
#	Package Variables
#**********************************************************************
my $m_bUseCIM;
my $m_strKey;
my $m_strKey2;

#**********************************************************************
#	Functions
#**********************************************************************

#**********************************************************************
# Name:			INIT
# Description:	Runs before the program executes
# Notes:
#**********************************************************************
INIT
{
	$m_bUseCIM = 0;

	# --cim - Specifies the Perl script contains CIM calls
	# --key - NSMI script key
	my %opts = ('cim' => \$m_bUseCIM, 'key:s' => \$m_strKey, 'key2:s' => \$m_strKey2);

	GetOptions(%opts);

} # INIT

#**********************************************************************
# Name:			END
# Description:	Runs when the program ends
# Notes:
#**********************************************************************
END
{
	# Clear the script key
	$m_bUseCIM = 0;
	$m_strKey = "";
	$m_strKey2 = "";

	undef($m_strKey);
	undef($m_strKey2);

} # END

#**********************************************************************
# Name:			EnumerateInstances
# Description:	Enumerates the instances of a given class
# Parameters:	scaler			- Class name
#				scaler			- Deep flag (T/F)
#				scaler			- Local only flag (T/F)
#				scaler			- Include qualifiers flag (T/F)
#				scaler			- Include class origin flag (T/F)
#				array ref		- Reference to property array
#				hash ref		- Reference to hash that receives instances
# Return:		unsigned int	- BCC error code
# Notes:
#**********************************************************************
sub EnumerateInstances($$$$$@%)
{
	my ($self, $class, $deep, $local, $qualifiers, $origin, $properties, $data) = @_;
	my $ret;
	my %params;

	# Clear the output data
	%$data = ();

    if (!$self->IsConfigurationOK())
	{
		return(ERR_INVALID_PARAMETER);
	} # if (!$self->IsConfigurationOK())

	$params{"OP"}			= WBEMOP_ENUMERATE_INSTANCES;
	$params{"KEY"}			= $m_strKey;
	$params{"KEY2"}			= $m_strKey2;
	$params{"CLASSNAME"}	= $class;
	$params{"DEEP"}			= $deep;
	$params{"LOCAL"}		= $local;
	$params{"QUALIFIERS"}	= $qualifiers;
	$params{"ORIGIN"}		= $origin;
	$params{"PROPERTIES"}	= $properties;

	$ret = &Novell_BCC::BCCSDK_PerlAPI(\%params, $data);

	$self->Finalize(\%params);

	return($ret);

} # EnumerateInstances

#**********************************************************************
# Name:			EnumerateInstanceNames
# Description:	Enumerates the instances of a given class
# Parameters:	scaler			- Class name
#				hash ref		- Reference to hash that receives instance names
# Return:		unsigned int	- BCC error code
# Notes:
#**********************************************************************
sub EnumerateInstanceNames($%)
{
	my ($self, $class, $data) = @_;
	my $ret;
	my %params;

	# Clear the output data
	%$data = ();
	
	if (!$self->IsConfigurationOK())
	{
		return(ERR_INVALID_PARAMETER);
	} # if (!$self->IsConfigurationOK())

	$params{"OP"}			= WBEMOP_ENUMERATE_INSTANCE_NAMES;
	$params{"KEY"}			= $m_strKey;
	$params{"KEY2"}			= $m_strKey2;
	$params{"CLASSNAME"}	= $class;

	$ret = &Novell_BCC::BCCSDK_PerlAPI(\%params, $data);

	$self->Finalize(\%params);

	return($ret);

} # EnumerateInstanceNames

#**********************************************************************
# Name:			InvokeMethod
# Description:	Enumerates the instances of a given class
# Parameters:	hash ref		- Reference to Object path hash
#				scaler			- Method name
#				hash ref		- Reference to method input paramemeter hash
#				hash ref		- Reference to hash that receives output
#								  parameter hash and return value
# Return:		unsigned int	- BCC error code
# Notes:
#**********************************************************************
sub InvokeMethod(%$%%)
{
	my ($self, $objectpath, $method, $methodparams, $data) = @_;
	my $ret;
	my %params;

	# Clear the output data
	%$data = ();
	
	if (!$self->IsConfigurationOK())
	{
		return(ERR_INVALID_PARAMETER);
	} # if (!$self->IsConfigurationOK())

	$params{"OP"}			= WBEMOP_INVOKE_METHOD;
	$params{"KEY"}			= $m_strKey;
	$params{"KEY2"}			= $m_strKey2;
	$params{"OBJECTPATH"}	= $objectpath;
	$params{"METHOD"}		= $method;
	$params{"PARAMS"}		= $methodparams;

	$ret = &Novell_BCC::BCCSDK_PerlAPI(\%params, $data);

	$self->Finalize(\%params);

	return($ret);

} # InvokeMethod

#**********************************************************************
# Name:			Associators
# Description:	Returns the associated instances of a given class
# Parameters:	hash ref		- Reference to Object path hash
#				scaler			- Result class filter
# 				scaler			- Associator class filter
#				hash ref		- Reference to hash that receives output
#								  parameter hash and return value
# Return:		unsigned int	- BCC error code
# Notes:
#**********************************************************************
sub Associators(%$$%)
{
	my ($self, $objectpath, $resultclass, $assocclass, $data) = @_;
	my $ret;
	my %params;

	# Clear the output data
	%$data = ();

	if (!$self->IsConfigurationOK())
	{
		return(ERR_INVALID_PARAMETER);
	} # if (!$self->IsConfigurationOK())

	$params{"OP"}			= WBEMOP_ASSOCIATE_INSTANCES;
	$params{"KEY"}			= $m_strKey;
	$params{"KEY2"}			= $m_strKey2;
	$params{"OBJECTPATH"}	= $objectpath;
	$params{"RESULTCLASS"}	= $resultclass;
	$params{"ASSOCCLASS"}	= $assocclass;
	
	$ret = &Novell_BCC::BCCSDK_PerlAPI(\%params, $data);

	$self->Finalize(\%params);

	return($ret);

} # Associators

#**********************************************************************
# Name:			Log
# Description:	Logs a message using the BCC logging facility
# Parameters:	scaler			- Message to log
# Return:		unsigned int	- BCC error code
# Notes:
#**********************************************************************
sub Log($)
{
	my ($self, $message) = @_;
	my $ret;
	my %params;
	my %data;	# Not used.  Place holder only.

	$params{"OP"}			= WBEMOP_LOG_MESSAGE;
	$params{"MESSAGE"}		= $message;

	if ($^O ne "NetWare")
	{
		$params{"LOGLEVEL"}	= 6;			# LOG_INFO - Linux (see syslog.h)
	}
	else
	{
		$params{"LOGLEVEL"}	= 2;			# LOG_INFO - NetWare (see _syslog.h)
	} # if ($^O ne "NetWare")

	$ret = &Novell_BCC::BCCSDK_PerlAPI(\%params, \%data);

	# Just in case
	$self->Finalize(\%params);

	return($ret);

} # Log

#**********************************************************************
# Name:			CreateCIMValue
# Description:	Creates a new Perl array that represents a CIM value
# Parameters:	scaler			- Value type
#				scaler			- Value
# Return:		array			- CIM value array
# Notes:
#**********************************************************************
sub CreateCIMValue($$)
{
	my ($self, $type, $value) = @_;
	my @cimvalue;

	$cimvalue[0] = 0;		# not an array
	$cimvalue[1] = $type;
	$cimvalue[2] = $value;

	return(@cimvalue);

} # CreateCIMValue

#**********************************************************************
# Name:			CreateCIMValueArray
# Description:	Creates a new Perl array that represents a CIM value
# Parameters:	scaler			- Value type
#				array ref		- Reference to value array
# Return:		array			- CIM value array
# Notes:
#**********************************************************************
sub CreateCIMValueArray($@)
{
	my ($self, $type, $value) = @_;
	my @cimvalue;

	$cimvalue[0] = 1;		# Array
	$cimvalue[1] = $type;
	push(@cimvalue, @$value);

	return(@cimvalue);

} # CreateCIMValueArray

#**********************************************************************
# Name:			AddNamedValue
# Description:	Adds a named value to a hash (object path, method
#				parameters, etc.)
# Parameters:	hash ref		- Reference to hash to add named value to
#				scaler			- Name of value
#				array			- Reference to CIM value to add, as
# 								  created by CreateCIMValue()
# Return:		None
# Notes:
#**********************************************************************
sub AddNamedValue(%$@)
{
	my ($self, $hash, $name, $value) = @_;

	$hash->{$name} = $value;

} # AddNamedValue

#**********************************************************************
# Name:			CreateObjectPath
# Description:	Creates a new Perl hash that represents a CIM object path.
# Parameters:	scaler			- Class name
# Return:		hash			- CIM object path
# Notes:		This creates the new hash with the class name in it.
# 				You must still use AddNamedValue() to add the keys
# 				to the object path hash.
#**********************************************************************
sub CreateObjectPath($)
{
	my ($self, $class) = @_;
	my %objectpath;

	$objectpath{"__CLASSNAME__"} = $class;

	return(%objectpath);

} # CreateObjectPath

#**********************************************************************
# Name:			CompareObjectPaths
# Description:	Compare to object paths
# Parameters:	hash ref		- Path 1
#				hash ref		- Path 2
# Return:		scalar			- 1 if equals, else 0
# Notes:		This will only compare the classname, namespace,
#				and keys (and not the hostname)
#**********************************************************************
sub CompareObjectPaths(%%)
{
	my ($self, $op1, $op2) = @_;

	eval
	{
		# Compare the class name
		if ($op1->{"__CLASSNAME__"} ne $op2->{"__CLASSNAME__"})
		{
			return(0);
		} # if ($op1->{"__CLASSNAME__"} ne $op2->{"__CLASSNAME__"})

		# Compare the name space
		if ($op1->{"__NAMESPACE__"} ne $op2->{"__NAMESPACE__"})
		{
			return(0);
		} # if ($op1->{"__NAMESPACE__"} ne $op2->{"__NAMESPACE__"})

		# Compare the number of keys
		if (scalar keys %$op1 ne scalar keys %$op2)
		{
			return(0);
		} # if (scalar keys %$op1 ne scalar keys %$op2)

		# Compare the keys
		foreach my $key (keys %$op1)
		{
			if ($op1->{$key} ne $op2->{$key})
			{
				return(0);
			} # if ($op1->{$key} ne $op2->{$key})
		} # foreach my $key (keys %$op1)
	}; # eval
	if ($@)
	{
		print("Error: $@\n");
		return(0);
	} # if ($@)

	return(1);

} # CompareObjectPaths

#**********************************************************************
# Name:			IsConfigurationOK
# Description:	Checks the parameters and configuration and makes sure
#				everything is OK.
# Parameters:	None
# Return:		true if OK.  false otherwise.
# Notes:
#**********************************************************************
sub IsConfigurationOK()
{
	if (0 == $m_bUseCIM)
	{
		return(0);
	} # if (0 == $m_bUseCIM)

	if (!defined($m_strKey) || $m_strKey eq "")
	{
		return(0);
	} # if (!defined($m_strKey) || $m_strKey eq "")

	if (!defined($m_strKey2) || $m_strKey2 eq "")
	{
		return(0);
	} # if (!defined($m_strKey2) || $m_strKey2 eq "")

	return(1);

} # IsConfigurationOK

#**********************************************************************
# Name:			Finalize
# Description:	Cleans up the param hash
# Parameters:	hash ref	- Reference to parameter hash
# Return:		None
# Notes:
#**********************************************************************
sub Finalize(%)
{
	my ($self, $hash) = @_;

	$hash->{"KEY"}	= "00000000:00000000:00000000:00000000";
	$hash->{"KEY2"}	= "00000000:00000000:00000000:00000000";

	%$hash = ();

} # Finalize

#**********************************************************************
# Name:			new
# Description:	Constructs a new CIMClient object
# Parameters:	None
# Return:		Reference to the new CIMClient object
# Notes:
#**********************************************************************
sub new()
{
	my $self = { };
	bless($self,);

	return($self);

} # new

1;

